%CRPolyMain
%
%Version 10.09.04 by Bernd Fischer


%Dimension of the system
N=100; 

%Eigenvalue bounds
a=.1; b=2; c=50; d=70;

%initial guess and solution
x_star=ones(N,1);
x0=rand(N,1);

%Stopping criteria
n_max=N;
tol=1e-15;

%Maximal polynomial degree and number of sample points
m=10;
dis=100;





matrix=menu('Choose matrix','Equidistant (one interval)', 'Chebyshev nodes',...
            'Laplace matrix', 'Equidistant (two intervals)');

if matrix == 1
%Equidistant1: equidistant eigenvalues in [a,b]
  A=linspace(a,b,N)'; 
  colour='b';
end

if matrix == 2
%Chebyshev: eigenvalues=Chebyshev nodes in [a,b]
  j = (0:1:(N-1))';   Cos=cos(pi*(2*j+1)/(2*N));
  A =  (a+b + (b-a)*Cos)/2;
  colour='r';
end

if matrix == 3
%Laplace: Laplace matrix in [a,b]
  n=sqrt(N); n=floor(n);
  h = 1 / (n + 1);
  lambda = zeros(n^2, 1);

  j = 0;
    for k = 1:n,
        for l = 1:n,
            j           = j + 1;
            lambda(j)   = 4 + 2 * cos(k*pi*h) + 2 * cos(l*pi*h);
        end;
    end

  alpha=min(lambda); beta=max(lambda);
  A=(b*(alpha-lambda)+a*(lambda-beta))/(alpha-beta);
  colour='g';
end

if matrix == 4
%Equidistant2: Equidistant in [a,b] u [c,d]
  i1 = floor( N * (b - a) / (b - a + d - c) );
  A = [ linspace( a, b, i1 ), linspace( c, d, N-i1 ) ]';
  colour='c';
end

matvec='mvdiag';
%Righthandside
rhs=feval( matvec, A, x_star );


%Cr iteration
[xCR, plotresCR, ploterrCR, alpha, beta, gamma] = ...
        cr_poly(A, rhs, x0, n_max, tol, matvec, x_star);


figure(1)
it=min([length(plotresCR)]);
plot(1:it,plotresCR(1:it),colour,'linewidth',2);


title('Convergence history for CR')
xlabel('Iterations,  equidistant1 (b), Chebyshev (r), Laplace (g), equidistant2 (c)')
ylabel('relative residual norm')


polynom=menu('Iteration polynomial ?', 'Yes', 'No');

if polynom == 1

%Compute polynomial
t=linspace(-1,max(A)+1,dis);
pm=zeros(m,dis);
for j=1:m
   pm(j,:)=orthopol(t,j,gamma,alpha,beta,1);
end


figure(2)
for j=2:m
   plot(0,1,'o')
   hold on
   plot([-1,max(A)+1],[0,0])
   plot([0,0],[-1.5,1.5])
   for k=1:N
      plot([A(k),A(k)],[-.1,.1],'g','linewidth',1.5)
   end
   plot(t,pm(j-1,:),'k','linewidth',2)
   plot(t,pm(j,:),'r','linewidth',2)
   drawnow, axis([-1,max(A)+1,-1.25,1.25])
   title('Eigenvalues and CR - polynomial')
   xlabel(['Polynomial degree: ', int2str(j-1),'/ ', int2str(j)])
   hold off
   choice=menu('Higher degree','    Yes   ','    No    '),
   if choice ~= 1, break, end
   clf, 
end
end

 

return